{% extends 'w3bsite/classes/apps/defaults/html/base.html' %}
{% load static %}
{% block content %}

<!-- CHECK INSTALLATION -->
<script type="text/javascript">
	toggle_top_bar("hide")
	set_header_path("maintenance/")
</script>

<!-- CONTENT -->
<div class="full_window_centered">

	<!-- LOGO -->
	<img src="{% static 'favicon.png' %}" width="45px" height="45px" style="display: block; margin: 0 auto; margin-bottom: 35px;">

	<!-- SIGN IN WIDGET -->
	<div class="widget_container"  style="">
		<div class="widget_child">
			<h1 class="title">
				Under Maintenance
			</h1>
			<p class="text">
				This domain is currently under maintenance, try again later or contact your system administrator.
			</p>
			<button class="disabled_button" onclick="" >
				Home
			</button>
		</div>
	</div>

	<!-- FOOTER -->
	<h1 class="text_reversed" style="font-size: 6px; font-weight: 400; margin-top: 35px;">
		© 2020 VanDenBerghInc. All Rights Reserved.
	</h1>

</div>

{% endblock %}