{% extends 'w3bsite/classes/apps/defaults/html/base.html' %}
{% load static %}
{% block content %}

<!-- CONTROL BAR -->
<div id="control_bar" style="padding: 20px 0px 20px 0px; overflow: scroll;">
	<div style="display: flex;">
		<div style="width: auto; display: flex; flex-direction: row;">
			<button id="control_bar.previous" class="button" onclick="" style="font-size: 12px; padding: 5px 20px 5px 20px; margin: 5px 5px 5px 5px; height: 25px; width: auto; min-width: none; white-space: nowrap;">
				<a href="/dashboard/home/" class="empty_a">
					Previous
				</a>
			</button>
		</div>
		<div style="flex: 1; display: flex; flex-direction: row-reverse;">
			<button id="control_bar.close" class="button" onclick="__toggle__('references')" style="font-size: 12px; padding: 5px 20px 5px 20px; margin: 5px 5px 5px 5px; height: 25px; width: auto; min-width: none; white-space: nowrap;">
				<a href="#" class="empty_a">
					Close
				</a>
			</button>
		</div>
	</div>
</div>

<!-- SHOW ERROR -->
<script type="text/javascript">
	show_error(
		// The error title.
		"{{ERROR.title}}",
		// The error message.
		"{{ERROR.message}}",
		// The error icon's path.
		"{% static '' %}{{ERROR.icon}}",
		// The left button's text.
		"Home",
		// The left button's function.
		function() {
			location.href = "/dashboard/home/"
		},
		// The right button's text.
		"{{ERROR.redirect_button}}",
		// The right button's function.
		function() {
			location.href = "{{ERROR.redirect}}"
		},
	)
</script>

{% endblock %}